/*
*  Rubrica
*  file: import.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

#include <gnome.h>

#include "rubrica.h"
#include "file.h"
#include "file_old.h"
#include "main.h"
#include "dialoghi.h"
#include "import.h"
#include "import_int.h"
#include "import_gcard.h"
#include "view.h"
#include "card.h"

ImportFrom importfrom;

static gchar *gcard_default="/.gnome/GnomeCard.gcrd";
static gchar *card_kde1="/.kde/share/apps/kab/addressbook.database";
static gchar *card_kde2="/.kde/share/apps/kab/";


void rubrica_import_from_init(GtkWidget *fentry, ImportFrom import)
{  
  importfrom = import; /* import default will be csv file */
      
  if(import == GCARD)
    gtk_widget_set_sensitive(GTK_WIDGET(fentry), FALSE);
}

void rubrica_import_from_set(ImportFrom import)
{
  importfrom = (ImportFrom) import;
}

ImportFrom rubrica_import_from_get(void)
{
  return (ImportFrom) importfrom;
}


void rubrica_import_data(void)
{
  Rubrica *rubrica;
  GtkWidget *fentry, *app;
  gchar *filename = NULL;
  gchar *buffer, *titolo = NULL;
  gint/* n,*/ page;
  
//  n=0;
  app     = rubrica_get_app();
  rubrica = rubrica_get_current_rubrica();
  fentry  = gnome_file_entry_gtk_entry(GNOME_FILE_ENTRY
				       (rubrica_import_get_file_entry()));

  filename = g_strdup(gtk_entry_get_text(GTK_ENTRY(fentry)));

  page = rubrica_card_add_page_view(NULL);
  
  switch (importfrom)
    {
    case GCARD:
      /* 	 if no name is set in gnome_entry, then 
		 load default gnomecard addressbook 
      */
      if (strcmp(filename,"") == 0)
	{
	  /* carico il file di default */
	  buffer = g_strdup(g_get_home_dir());
	  
	  filename = g_strdup_printf("%s%s",buffer,gcard_default);

	  if (!g_file_exists(filename))
	    {
	      gchar *str = g_strdup_printf(_("Error.\n%s don't exist."), filename);
	      
	      rubrica_dialog_error(str);
	      g_free(str);
	    }

	  rubrica_import_open_file(rubrica, filename);
	  titolo = _("Imported from GnomeCard");
//	  n = rubrica_view_get_indirizzi();
	}          
      else	
	/* import gnomecard file */
	{
	  rubrica_import_open_file(rubrica, filename);
//	  n = rubrica_view_get_indirizzi();		  
	}
      break;
      
    case KCARD:
      /* kab (kab 2.x) has a default database */
      buffer =  g_strdup(g_get_home_dir());
      
      filename = g_strdup_printf("%s%s",buffer,card_kde1);
      rubrica_import_open_file(rubrica, filename);
      titolo = _("Imported from Kab");
//      n = rubrica_view_get_indirizzi();
      break;

    case RUBRICA:
      if (strcmp(filename, "") == 0)
	{
	  rubrica_dialog_error(_("Error.\nYou must select a file before.\n"));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else
      /* apri il file e leggilo */
	{
	  g_print("\nimport %s", filename);
	  rubrica_import_open_old_rubrica_file(rubrica, filename);      
	}
      break;

    case CSV:
    default:
      if (strcmp(filename,"") == 0)
	{
	  rubrica_dialog_error(_("Error.\nYou must select a file before."));
	  g_error(_("Error.\nYou must select a file before.\n"));
	  return;
	}
      else 
	{
	  g_print("file: %s", filename);
	  rubrica_import_open_file(rubrica, filename);      
	  titolo = _("Imported from Netscape csv");	  
//	  n = rubrica_view_get_indirizzi();
	}
      break;      
    }
  
  rubrica_set_path(rubrica, g_dirname(filename));
  rubrica_set_name(rubrica, g_basename(filename));
  rubrica_set_new(rubrica, FALSE);
  rubrica_set_empty(rubrica, FALSE);
  rubrica_set_modified(rubrica, TRUE);

  rubrica_view_set_items(rubrica_get_items(rubrica));

  /* visualizzo il numero di indirizzi importati */
//  rubrica_view_set_indirizzi(n);
}


void rubrica_import_open_file(Rubrica *rubrica, gchar *file)
{
  FILE *fp;

  if ((fp=fopen(file, "r")) == NULL)
    {
      rubrica_dialog_error_loading_file (file, RE_IMPORT_FILE);

      return;
    }

  while (!feof(fp))
    {
      gchar buffer[124];

      while (fgets(buffer, 124, fp))
	{	  
	  rubrica_import_gcard_decode_buffer(rubrica, buffer);
	}
    }
}

void rubrica_export_data(void)
{
}


gint rubrica_import_open_old_rubrica_file(Rubrica *rubrica, gchar *fname)
{
  RubricaErrorType err;
  gboolean loaded;
  
  loaded = rubrica_file_load_xmlfile(fname, &err);

  if (!loaded)
    {
      rubrica_dialog_error_loading_file(fname, err);      

      return 0;
    }
  
  return (gint) rubrica_view_get_indirizzi();    
}



