/*
*  program name: Rubrica <http://digilander.iol.it/nfragale/download.html>
*  file: prefer-configure.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <gconf/gconf-client.h> 
#include <gconf/gconf-schema.h>
#include <libxml/tree.h>
#include <time.h>

#include "main.h"
#include "dialoghi.h"
#include "prefer_cb.h"
#include "prefer_int.h"
#include "prefer.h"
#include "prefer-configure.h"
#include "rubrica.h"

#undef DEBUG

gboolean have_errors = FALSE;

void rubrica_preferences_reset_errors(void)
{
  have_errors = FALSE;
}

gboolean rubrica_preferences_there_are_errors(void)
{
  return have_errors;
}

void rubrica_preferences_configure_radio(GConfClient *client, int count, ...)
{
  va_list ap;
  GtkWidget *radio;
  gchar *key;
  gboolean bool = FALSE;
  GConfValue *val_key;
  gint i = 0;

  va_start(ap, count);
  
  while (i < count)
    {
      radio = (GtkWidget *) va_arg(ap, GtkWidget *);
      key   = (gchar *) va_arg(ap, gchar *);

      /*     get the value of key1 stored in gconf database
       */  
      val_key = gconf_client_get(client, key, NULL);
      
      /*    if rubrica was never run before, the value for key1 is NULL, so
	    i will read default value stored in schema.
	    if value isn't NULL and his type is BOOL then i read this value
      */
      if (val_key == NULL || val_key->type != GCONF_VALUE_BOOL)    
	val_key = gconf_client_get_default_from_schema(client, key, NULL);      
      
      if (!val_key)
	{
	  rubrica_dialog_error_in_preferences_key(key);
	  
	  have_errors = TRUE;
	  
	  if (strcmp(key, RUBRICA_KEY_ALPHA_ORDER) == 0)
	    bool = rubrica_preferences_get_alphabetic_order();
	  else if (strcmp(key, RUBRICA_KEY_COMPLETE) == 0)
	    bool = rubrica_preferences_get_complete_search();
	  else if (strcmp(key, RUBRICA_KEY_ITALIAN) == 0)
	    bool = rubrica_preferences_get_use_italian_dictionary();
	  
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), bool);     
	}
      else
	{
	  bool = gconf_client_get_bool(client, key, NULL); 
	  
	  if (bool)
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE);
	  else
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio), TRUE); 
	} 
 
      i++;
    }
  va_end(ap); 
}

 
void       
rubrica_preferences_configure_radio_dictionary(GConfClient *client,    
					       GtkWidget *radio1,  /* external dic */    
					       GtkWidget *radio2,  /* default  dic */       
					       gchar *key1,        /* external key */
					       gchar *key2)        /* default  key */ 
{ 
  GConfValue *val_key;
  
  GtkWidget *prefbox;
  GtkWidget *radio_italian;
  GtkWidget *radio_english;
  GtkWidget *entry;
  GtkWidget *notebook;
  GtkWidget *label_default;
  GtkWidget *label_language;
  GtkWidget *label_io;
  GtkWidget *label_dict;
  GtkWidget *load;
  GtkWidget *save;
  
  gboolean bool;

#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_radio_dictionary");
  g_print("\nradio1: %p\nradio2: %p\nclient: %p\nkey1: %s\nkey2: %s", 
	  radio1, radio2, client, key1, key2);
#endif

  /*    radio1 is the button for external dictionary, while 
	radio2 is the button for default dictionary
  */    
  prefbox = rubrica_preferences_get_preferencesbox();

  radio_italian  = gtk_object_get_data(GTK_OBJECT(prefbox), RUBRICA_KEY_ITALIAN);
  radio_english  = gtk_object_get_data(GTK_OBJECT(prefbox), RUBRICA_KEY_ENGLISH);
  label_default  = gtk_object_get_data(GTK_OBJECT(prefbox), "lbl_default_dict");
  entry          = gtk_object_get_data(GTK_OBJECT(prefbox), RUBRICA_KEY_USER_DICTIONARY);
  label_language = gtk_object_get_data(GTK_OBJECT(prefbox), "lbl_language");
  notebook       = gtk_object_get_data(GTK_OBJECT(prefbox), "notebook_dict");
  label_io       = gtk_object_get_data(GTK_OBJECT(prefbox), "lbl_io");
  label_dict     = gtk_object_get_data(GTK_OBJECT(prefbox), "lbl_dict_name");
  load           = gtk_object_get_data(GTK_OBJECT(prefbox), "load");
  save           = gtk_object_get_data(GTK_OBJECT(prefbox), "save");

  /*     get the value of key1 (external dictionary) stored in gconf database
  */
  val_key = gconf_client_get(client, key1, NULL);

  if (val_key == NULL || val_key->type != GCONF_VALUE_BOOL)    
    val_key = gconf_client_get_default_from_schema(client, key1, NULL);      

  if (!val_key)
    {
      rubrica_dialog_error_in_preferences_key(key1);
      
      have_errors = TRUE;

      bool = rubrica_preferences_get_use_user_dictionary();
    }
  else
    bool = gconf_client_get_bool(client, key1, NULL); 
    
  if (bool)    /* use external dictionary (user dictionary) */
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio1), TRUE);
      gtk_widget_set_sensitive(radio_italian, FALSE);
      gtk_widget_set_sensitive(radio_english, FALSE);
    }      
  else        /* use one of default dictionary */
    {
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (radio2), TRUE);  
      gtk_widget_set_sensitive(radio_italian, TRUE);
      gtk_widget_set_sensitive(radio_english, TRUE);
    }
  
  gtk_widget_set_sensitive(label_default, !bool);

  gtk_widget_set_sensitive(entry, bool);
  gtk_widget_set_sensitive(label_language, bool);
  gtk_widget_set_sensitive(notebook, bool);
  gtk_widget_set_sensitive(label_io, bool);
  gtk_widget_set_sensitive(label_dict, bool);
  gtk_widget_set_sensitive(load, bool); 
  gtk_widget_set_sensitive(save, bool);      
}


void 
rubrica_preferences_configure_check(GConfClient *client, GtkWidget *check,  gchar *key)
{
  GConfValue *key_value;

#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_check");
  g_print("\ncheck: %p\nclient: %p\nkey: %s", check, client, key);
#endif

  /*     get the value of key stored in gconf database
  */
  key_value = gconf_client_get(client, key, NULL);

  /*    if rubrica was never run before, the value for key is NULL, so
	i will read default value stored in schema.
	if value isn't NULL and his type is BOOL then i read this value
  */
  if (key_value == NULL || key_value->type != GCONF_VALUE_BOOL)
    key_value = gconf_client_get_default_from_schema(client, key, NULL);      
  
  if (!key_value)
    {
      gboolean bool = FALSE;

      rubrica_dialog_error_in_preferences_key(key);

      have_errors = TRUE;
            
      if (strcmp(key, RUBRICA_KEY_SHOW) == 0)
	bool = rubrica_preferences_get_show_path();
      
      else if (strcmp(key, RUBRICA_KEY_BACKUP) == 0)
	bool = rubrica_preferences_get_do_backup();
      
      else if (strcmp(key, RUBRICA_KEY_INCREASE) == 0)
	bool = rubrica_preferences_get_increasing_order();
      
      else if (strcmp(key, RUBRICA_KEY_EXIT) == 0)
	bool = rubrica_preferences_get_exit();
/*      
      else if (strcmp(key, RUBRICA_KEY_TIPS) == 0)
	bool = rubrica_preferences_get_tips();    
*/
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), bool);      
    }
  else
    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check),
				  gconf_value_get_bool(key_value));
}


void rubrica_preferences_configure_check_radio(GConfClient *client,
					       GtkWidget *check, 
					       GtkWidget *radio1, 
					       GtkWidget *radio2,		        
					       gchar *checkkey,
					       gchar *typekey)
{
  gboolean bool = FALSE;
  GConfValue *checkval = NULL;
  GConfValue *radioval = NULL;  

#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_check_radio");
  g_print("\ncheck: %p\nardio 1: %p\nradio 2: %p\nclient: %p\ncheck_key: %s\ntype key: %s", check, radio1, radio2, client, checkkey, typekey);
#endif

  /* get the value stored in gconf database for the key "check_key"
  */
  checkval = gconf_client_get(client, checkkey, NULL);

  /* if the database hasn't data for the key, then get the default value 
     from the schema
  */  
  if (checkval == NULL || checkval->type != GCONF_VALUE_BOOL)
    checkval = gconf_client_get_default_from_schema(client, checkkey, NULL); 
    
  if (!checkval)
    {
      rubrica_dialog_error_in_preferences_key(checkkey);

      have_errors = TRUE;

      /* use test and not direct chech, for future expansion */
      /* eseguo un test sul valore della chiave e non chiamo direttamente
	 la funzione per evitare di modificare il codice in future 
	 espansioni
      */
      if (strcmp(checkkey, RUBRICA_KEY_BACKUP) == 0)
	bool = rubrica_preferences_get_do_backup();
    }
  else
    /* now I have a key value, and i can extract the boolean value  */
    bool = gconf_value_get_bool(checkval); 
  
  if (bool)
    {
      gboolean type;

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
      gtk_widget_set_sensitive (radio1, TRUE);
      gtk_widget_set_sensitive (radio2, TRUE);
      
      radioval = gconf_client_get(client, typekey, NULL); 
      
      if (radioval == NULL || radioval->type != GCONF_VALUE_BOOL)  	
	radioval = gconf_client_get_default_from_schema(client, typekey, NULL); 
      
      if (!radioval)
	{
	  rubrica_dialog_error_in_preferences_key(typekey);
	  
	  have_errors = TRUE;
	  
	  type = rubrica_preferences_get_backup_type(typekey);
	}
      else
	{
	  /* get value stored in the key */
	  type = gconf_value_get_bool(radioval);    
	  
	  if (strcmp(typekey, RUBRICA_KEY_BACKUP_TYPE_INC) == 0)
	    {
	      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio1), type);
	      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio1), !type);
	    }
	  
	  if (strcmp(typekey, RUBRICA_KEY_BACKUP_TYPE_BAK) == 0)
	    {
	      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio1), !type);
	      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio1), type);
	    }
	}
    }	  
  else
    {
      gtk_widget_set_sensitive (radio1, FALSE);
      gtk_widget_set_sensitive (radio2, FALSE);
    }    
}


void 
rubrica_preferences_configure_checked_entry(GConfClient *client,
					    GtkWidget *check, 
					    GtkWidget *gnome_entry,
					    GtkWidget *entry, 				    
					    gchar *check_key,  
					    gchar *gentry_key)
{
  gboolean bool = FALSE;
  GConfValue *checkval = NULL;
  GConfValue *gentryval = NULL;  

#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_checked_entry");
  g_print("\ncheck: %p\ngnome entry: %p\nentry: %p\nclient: %p\ncheck_key: %s\ngentry_key: %s", check, gnome_entry, entry, client, check_key, gentry_key);
#endif

  /* get the value stored in gconf database for the key "check_key"
  */
  checkval = gconf_client_get(client, check_key, NULL);

  /* if the database hasn't data for the key, then get the default value 
     from the schema
  */  
  if (checkval == NULL || checkval->type != GCONF_VALUE_BOOL)
    checkval = gconf_client_get_default_from_schema(client, check_key, NULL); 
    
  if (!checkval)
    {
      rubrica_dialog_error_in_preferences_key(check_key);

      have_errors = TRUE;

      if (strcmp(check_key, RUBRICA_KEY_HAVE_BOOK) == 0)
	bool = rubrica_preferences_get_have_default_addressbook();
      
      else if (strcmp(check_key, RUBRICA_KEY_HAVE_FOLDER) == 0)
	bool = rubrica_preferences_get_have_default_folder();
    }
  else
    /* now I have a key value, and i can extract the boolean value  */
    bool = gconf_value_get_bool(checkval); 
  
  if (bool)
    {
      gchar *buffer = NULL;

      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (check), TRUE);
      gtk_widget_set_sensitive (gnome_entry, TRUE);
      
      /* again get the value stored in gconf database for the key "gentry_key"
       */
      gentryval = gconf_client_get(client, gentry_key, NULL); 
      
      if (gentryval == NULL || gentryval->type != GCONF_VALUE_STRING)  	
	gentryval = gconf_client_get_default_from_schema(client, gentry_key, NULL); 
      
      if (!gentryval)
	{
	  rubrica_dialog_error_in_preferences_key(gentry_key);
	  
	  have_errors = TRUE;
	  
	  if (strcmp(gentry_key, RUBRICA_KEY_BOOK) == 0)
	    buffer = rubrica_preferences_get_book();
	  
	  else if (strcmp(gentry_key, RUBRICA_KEY_FOLDER) == 0)
	    buffer = rubrica_preferences_get_default_folder();	  
	}
      else
	/* get value stored in the key */
	buffer = gconf_value_get_string(gentryval);   

      if (buffer)
	{
	  gtk_entry_set_text (GTK_ENTRY (entry), buffer);

	  g_free(buffer);     
	}
      else
	gtk_entry_set_text (GTK_ENTRY (entry), "");      
    }
  else
    {
      gtk_entry_set_text (GTK_ENTRY (entry), "");
      gtk_widget_set_sensitive(gnome_entry, FALSE);
    }  
}


void rubrica_preferences_configure_size_spin(GConfClient *client, GtkWidget *spin, gchar *key)
{
  GConfValue *gconf_value = NULL;
  gint rate = 0; 

  gconf_value = gconf_client_get(client, key, NULL);
  if (gconf_value == NULL || gconf_value->type != GCONF_VALUE_INT)
    gconf_value = gconf_client_get_default_from_schema(client, key, NULL); 
  
  rate = gconf_value_get_int(gconf_value);
  gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), rate);     
}


void rubrica_preferences_configure_spin(GConfClient *client,
					GtkWidget *check, 
					GtkWidget *spin, 
					gchar *check_key, 
					gchar *rate_key)
{
  GtkWidget *prefbox  = NULL;      /* ptr to preferences widget */
  GtkWidget *spin_box = NULL;      /* box that contains spin and labels */

  GConfValue *checkval = NULL;     /* value stored for the check button */
  GConfValue *rateval  = NULL;     /* value stored for the spin button  */

  gint rate     = 0;               /* zip_rate if compression, time if autosave */
  gboolean bool = FALSE;

#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_spin");
  g_print("\ncheck: %p\nspin: %p\nclient: %p\ncheck_key: %s\nrate_key: %s", 
	  check, spin, client, check_key, rate_key);
#endif

  prefbox = rubrica_preferences_get_preferencesbox();
  spin_box = gtk_object_get_data(GTK_OBJECT(check), "spin_box");

  /* get the key related to SPIN_KEY then get the key value stored
     in gconf database 
  */
  checkval = gconf_client_get(client, check_key, NULL);

  /* if the database hasn't data for the key, then get the default value 
     from the schema
  */
  if (checkval == NULL || checkval->type != GCONF_VALUE_BOOL)
    checkval = gconf_client_get_default_from_schema(client, check_key, NULL); 
  
  if (!checkval)
    {
      rubrica_dialog_error_in_preferences_key(check_key);

      have_errors = TRUE;
      
      if (strcmp(check_key, RUBRICA_KEY_COMPRESS) == 0)
	bool = rubrica_preferences_get_compress();
      
      if (strcmp(check_key, RUBRICA_KEY_AUTOSAVE) == 0)
	bool = rubrica_preferences_get_autosave();      
    }
  else
    /* now I have a key value, and i can extract the boolean value */
    bool = gconf_value_get_bool(checkval); 
  
  /* check the button */
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (check), bool);

  if (bool)
    {
      /* make sensitive the spin button and the related labels */
      gtk_widget_set_sensitive(spin_box, TRUE);
      gtk_widget_set_sensitive(spin, TRUE);
      
      /* extract the "compression or time" rate from rate_key value 
	 or read default from schema
      */
      rateval = gconf_client_get(client, rate_key, NULL);
      
      if (rateval == NULL || rateval->type != GCONF_VALUE_INT)
	rateval = gconf_client_get_default_from_schema(client, rate_key, NULL); 
      
      if (!rateval)
	{
	  rubrica_dialog_error_in_preferences_key(rate_key);
	  
	  have_errors = TRUE;

	  if (strcmp(check_key, RUBRICA_KEY_RATE) == 0)
	    rate = rubrica_preferences_get_compress_rate();
	  
	  if (strcmp(check_key, RUBRICA_KEY_WAIT_TIME) == 0)
	    rate = rubrica_preferences_get_time_rate();
	}

      else
	/* get the compression_rate/wait_time 
	   and put it into spin button	 
	*/
	rate = gconf_value_get_int(rateval); 
      
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), rate);     
    }
  else
    {
      /* user don't want a compressed addressbook 
	 (automatic save), set sensitive of
	 spin button and related label to false
      */
      gtk_widget_set_sensitive(spin, FALSE);	  
      gtk_widget_set_sensitive(spin_box, FALSE);
    }
}

void rubrica_preferences_configure_entry(GConfClient *client, GtkWidget *entry, gchar *key)
{
  GConfValue *val_key;
  gchar *str;

#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_entry");
  g_print("\nentry: %p\nclient: %p\nkey: %s", 
	  entry, client, key);
#endif

  val_key = gconf_client_get(client, key, NULL);

  if (val_key == NULL || val_key->type != GCONF_VALUE_STRING)
    val_key = gconf_client_get_default_from_schema(client, key, NULL); 
  
  if (val_key)
    {
      str = gconf_client_get_string(client, key, NULL); 
      
      if (str)
	{
	  gtk_entry_set_text(GTK_ENTRY(entry), g_basename(str));
	  
	  g_free(str);
	}
    }
  else
    gtk_entry_set_text(GTK_ENTRY(entry), "");
}


void rubrica_preferences_configure_label(GConfClient *client, GtkWidget *label, gchar *key)
{
  GConfValue *val_key;
  gchar *str;

#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_label");
  g_print("\nlabel: %p\nclient: %p\nkey: %s", 
	  label, client, key);
#endif

  val_key = gconf_client_get(client, key, NULL);

  if (val_key == NULL || val_key->type != GCONF_VALUE_STRING)
    val_key = gconf_client_get_default_from_schema(client, key, NULL); 
  
  if(val_key)
    {
      str = gconf_client_get_string(client, key, NULL); 
      
      if (str)
	{
	  gtk_label_set_text(GTK_LABEL(label), str);
	}
     else
     	gtk_label_set_text(GTK_LABEL(label), "");
    }
  else
    gtk_label_set_text(GTK_LABEL(label), "");
}

void rubrica_preferences_configure_combo_entry(GConfClient *client, GtkWidget *entry, 
					       gchar *key)
{
  GConfValue *val_key;
  gint value;
  gchar *str;
  
#ifdef DEBUG
  g_print("\nrubrica_preferences_configure_combo");
  g_print("\ncombo: %p\nclient: %p\nkey: %s", combo, client, key);
#endif
  
  val_key = gconf_client_get(client, key, NULL);
  
  if (val_key == NULL || val_key->type != GCONF_VALUE_INT)
    val_key = gconf_client_get_default_from_schema(client, key, NULL); 
  
  if(val_key)
    {
      value = gconf_client_get_int(client, key, NULL); 
      
      switch (value)
	{
	case 0:
	  str = RUBRICA_SEARCH_CARD;
	  break;
	  
	case 1:
	  str = RUBRICA_SEARCH_FIRST_NAME;
	  break;
	  
	case 2:
	  str = RUBRICA_SEARCH_LAST_NAME;
	  break;
	  
	case 3:
	  str = RUBRICA_SEARCH_PROFESSION;
	  break;
	  
	case 4:
	  str = RUBRICA_SEARCH_CITY;
	  break;
	  
	case 5:
	  str = RUBRICA_SEARCH_COUNTRY; 
	  break;
	  
	case 6:
	  str = RUBRICA_SEARCH_WEB_ADDRESS;
	  break;
	  
	case 7: 
	  str = RUBRICA_SEARCH_EMAIL_ADDRESS;
	  break;
	  
	case 8:
	  str = RUBRICA_SEARCH_TELEPHONE;
	  break;
	  
	default:
	  str = RUBRICA_SEARCH_CARD;
	  break;
	}      
      
      gtk_entry_set_text (GTK_ENTRY(entry), str);    
    }
  else
    gtk_entry_set_text (GTK_ENTRY(entry), _(RUBRICA_SEARCH_CARD));
}

