#include <gnome.h>
#include <gconf/gconf-client.h>

#include "prefer.h"
#include "autosave.h"
#include "prefer_int.h"
#include "prefer-notify.h"
#include "rubrica.h"

void rubrica_notify_path_config(GConfClient *client, GtkWidget *label, gchar *key)
{
  GConfValue *show_path = NULL;  
  gchar *str;
  guint notify_id;

  show_path = gconf_client_get(client, key, NULL);  

  if (show_path != NULL && 
      show_path->type == GCONF_VALUE_BOOL &&
      gconf_value_get_bool(show_path))
    {          
      GConfValue *book = NULL; 
      
      book = gconf_client_get(client, RUBRICA_KEY_LAST_BOOK, NULL);
      
      if (book != NULL && book->type == GCONF_VALUE_STRING)
	{		  	  
	  str = g_dirname(gconf_value_get_string(book));
	  
	  gtk_label_set_text(GTK_LABEL(label), str);
	  
	  g_free(str);	      
	}
    }    
  else    
    gtk_label_set_text(GTK_LABEL(label), "");
  
  notify_id = gconf_client_notify_add(client, 
				      key, 
				      rubrica_notify_path_changes, 
				      label, 
				      NULL, 
				      NULL);

   g_object_set_data(G_OBJECT(client), "show_path_notify", GUINT_TO_POINTER(notify_id));
}

void rubrica_notify_path_changes(GConfClient *client, guint cnxn_id, 
				 GConfEntry *entry, gpointer data)
{  
  GConfValue *book = NULL; 
  gchar *str;
  
  book = gconf_entry_get_value (entry);
  
  if (book != NULL && book->type == GCONF_VALUE_STRING)
    {		  	        
      str = g_dirname(gconf_value_get_string(book));
      
      gtk_label_set_text(GTK_LABEL(data), str);
      
      g_free(str);	      
    }
  else
    gtk_label_set_text(GTK_LABEL(data), "");  
}


void rubrica_notify_check_config(GConfClient *client, GtkWidget *check, gchar *key)
{
  GConfValue *checkval = NULL;  
  guint notify_id;

  checkval = gconf_client_get(client, key, NULL);  

  if (checkval != NULL && 
      checkval->type == GCONF_VALUE_BOOL &&
      gconf_value_get_bool(checkval))
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), TRUE);              
  else    
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), FALSE);              
  
  notify_id = gconf_client_notify_add(client, key, rubrica_notify_check_changes, 
				      check, NULL, NULL);

  if (strcmp(key, RUBRICA_KEY_SENSITIVE) == 0)
    gtk_object_set_data(GTK_OBJECT(client), "check_sensitive", GUINT_TO_POINTER(notify_id));
  else
    gtk_object_set_data(GTK_OBJECT(client), "check_incomplete",	GUINT_TO_POINTER(notify_id));   
}

void rubrica_notify_check_changes(GConfClient *client, guint cnxn_id, 
				  GConfEntry *entry, gpointer data)
{  
  GtkWidget *check = data;

  if (entry->value != NULL && 
      entry->value->type == GCONF_VALUE_BOOL)      
    {
      gboolean bool;

      bool = gconf_value_get_bool(entry->value);
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), bool);
    }
}


void rubrica_notify_autosave_time_changes(GConfClient *client, guint cnxn_id, 
					  GConfEntry *entry, gpointer data)
{
  /*     rubrica_autosave_time_changes is called when value in spin button 
	 is changed. Spin button is sensible only if check button is setted.
	 So if we are here, we have a timeout_tag and have_timeout is true
  */
  GtkWidget *spin = data;
  gint time = 0;
  
  time = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin));
  rubrica_autosave_restart(time);
  g_print("\nrubrica_notify_autosave_time_changes");
}

void rubrica_notify_column_config(GConfClient *client, GtkTreeViewColumn *column, gchar *key)
{
  gboolean bool;

  bool = gconf_client_get_bool(client, key, NULL);
  gconf_client_notify_add(client, key, rubrica_notify_column_changes, column, NULL, NULL);
  
  gtk_tree_view_column_set_visible(column, bool);  
}

void rubrica_notify_column_changes(GConfClient *client, guint cnxn_id, 
				   GConfEntry *entry, gpointer data)
{
  GtkTreeViewColumn *column = (GtkTreeViewColumn *) data;
  gboolean bool;

  bool = gconf_value_get_bool(entry->value);
  gtk_tree_view_column_set_visible(GTK_TREE_VIEW_COLUMN(column), bool);    
}

