/*
*  program name: Rubrica <http://digilander.iol.it/nfragale/download.html>
*  file: prefer.c
*  
*  Copyright (C) 2000-2001 Nicola Fragale <nicolafragale@libero.it>
*
*  This program is free software; you can redistribute it and/or modify
*  it under the terms of the GNU General Public License as published by
*  the Free Software Foundation; either version 2 of the License, or
*  (at your option) any later version.
*
*  This program is distributed in the hope that it will be useful,
*  but WITHOUT ANY WARRANTY; without even the implied warranty of
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*  GNU General Public License for more details.
*
*  You should have received a copy of the GNU General Public License
*  along with this program; if not, write to the Free Software
*  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/


#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gnome.h>
#include <gconf/gconf-client.h>
#include <gconf/gconf-schema.h>
#include <libxml/tree.h>
#include <time.h>

#include "main.h"
#include "error.h"
#include "prefer.h"
#include "prefer_cb.h"
#include "prefer_int.h"
#include "prefer-revert.h"
#include "rubrica.h"


void
rubrica_preferences_revert(GtkWidget* dialog)
{
  GConfChangeSet* cs;
  GConfChangeSet* revert_cs;
  GConfClient* client;

  cs        = gtk_object_get_data(GTK_OBJECT(dialog), "changeset");
  client    = gtk_object_get_data(GTK_OBJECT(dialog), "client"); 

  /*      User wants to revert to old values. 
	  So I must Get the ptr to old values
  */
  revert_cs = gtk_object_get_data(GTK_OBJECT(dialog), "revert_cs");

  if (revert_cs == NULL)
    return;  /* happens on cancel, if no apply has been done */
  
  /*      discard any pending changes so "apply" won't do anything  
  */
  gconf_change_set_clear(cs);

  /*      FALSE so we don't remove committed stuff from the revert set 
  */
  gconf_client_commit_change_set(client, revert_cs, FALSE, NULL);

  /*      Set the prefs dialog contents back to the new values 
  */
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW);
  
  rubrica_preferences_revert_checked_entry(dialog, revert_cs, 
					   RUBRICA_KEY_HAVE_BOOK,
					   RUBRICA_KEY_BOOK);
  
  rubrica_preferences_revert_checked_entry(dialog, revert_cs,
					   RUBRICA_KEY_HAVE_FOLDER,
					   RUBRICA_KEY_FOLDER);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_BACKUP);
  
  rubrica_preferences_revert_radio_buttons(dialog, revert_cs, 
					   RUBRICA_KEY_BACKUP_TYPE_INC, 
					   RUBRICA_KEY_BACKUP_TYPE_BAK);

  rubrica_preferences_revert_spin_button(dialog, revert_cs, 
					 RUBRICA_KEY_COMPRESS,
					 RUBRICA_KEY_RATE); 

  rubrica_preferences_revert_spin_button(dialog, revert_cs, 
					 RUBRICA_KEY_AUTOSAVE,
					 RUBRICA_KEY_WAIT_TIME); 
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_INCREASE);

  rubrica_preferences_revert_radio_buttons(dialog, revert_cs,  
					   RUBRICA_KEY_ALPHA_ORDER,
					   RUBRICA_KEY_DATE_ORDER);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 	 
					  RUBRICA_KEY_SENSITIVE);
  
  rubrica_preferences_revert_radio_buttons(dialog, revert_cs,  
					   RUBRICA_KEY_INCOMPLETE,
					   RUBRICA_KEY_INCOMPLETE);

  rubrica_preferences_revert_entry(dialog, revert_cs, 
				   RUBRICA_KEY_SEARCH);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs,
					  RUBRICA_KEY_EXIT);

  rubrica_preferences_revert_check_button(dialog, revert_cs,
					  RUBRICA_KEY_TIPS);

  rubrica_preferences_revert_radio_buttons(dialog, revert_cs,
					   RUBRICA_KEY_EXTERNAL,
					   RUBRICA_KEY_DEFAULT);
					   
  rubrica_preferences_revert_radio_buttons(dialog, revert_cs, 	
					   RUBRICA_KEY_ITALIAN,
					   RUBRICA_KEY_ENGLISH);

  rubrica_preferences_revert_entry(dialog, revert_cs, 
				   RUBRICA_KEY_USER_DICTIONARY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_CARD);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_FIRST_NAME);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_LAST_NAME);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_PROFESSION);

  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_CITY);

  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_COUNTRY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_WEB);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_EMAIL);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_TELEPHONE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_COMPANY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_CARD_COLUMN_ASSIGMENT);

  /*  ***********************************************************************  */
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_MIDDLE_NAME);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_NAME_PREFIX);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_TITLE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_BIRTHDAY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_STREET);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_STREET_NUM);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_ZIP_CODE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_PROVINCE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_STATE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_COMPANY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_STREET);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_STREET_NUM);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_ZIP_CODE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_CITY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_PROVINCE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_COUNTRY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_WEB);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_EMAIL);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_OPERATOR);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_FAX);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_GREEN);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_CUSTOMER_CARE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_JOB_NOTE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_ASSIGMENT);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_ORG);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_DEP);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_SUB_DEP);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_SECRETARY);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_SECR_PHONE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_MARRIED);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_SPOUSE);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_CHILDREN);
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_SPOUSE_BIRTH);
  
/*  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_CHILD_NOTE);
*/
  
  rubrica_preferences_revert_check_button(dialog, revert_cs, 
					  RUBRICA_KEY_SHOW_DATA_NOTE);
  

  /* Update sensitivity of the dialog buttons */
  rubrica_preferences_update_sensitivity(dialog);
}


/*  
 */
void rubrica_preferences_revert_check_button(GtkWidget *dialog, GConfChangeSet* cs, 
					     gchar* check_key)
{
  GConfValue  *value = NULL;
  GConfClient *client;
  GtkWidget   *check;
		  
  check  = gtk_object_get_data(GTK_OBJECT(dialog), check_key);
  client = gtk_object_get_data(GTK_OBJECT(dialog), "client");

  if (gconf_change_set_check_value(cs, check_key, &value))
    {    
      /*   If values for key1 and key2 was changed, but changes wasn't apply 
	   then no values have been stored into database for key1 and key2, 
	   so if user wants to revert, he can revert only to default value 
	   stored into schema.
       */   
      if (value == NULL)
        {
          value = gconf_client_get_default_from_schema(client, check_key, NULL);
  
	  if (value && value->type == GCONF_VALUE_BOOL)	    
	    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check),
					  gconf_value_get_bool(value));
	}
    }
  else  /*  revert to previous value  */
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check),
				 gconf_value_get_bool(value));
}  


void rubrica_preferences_revert_radio_buttons(GtkWidget *dialog, GConfChangeSet* cs, 
					      gchar* radio_key1,gchar* radio_key2)
{
  GConfValue* value1 = NULL;
  GConfValue* value2 = NULL;
  GConfClient* client;
  gboolean check1, check2;
  
  client = gtk_object_get_data(GTK_OBJECT(dialog), "client");

  check1 = gconf_change_set_check_value(cs, radio_key1, &value1);
  check2 = gconf_change_set_check_value(cs, radio_key2, &value2);

  if (check1 || check2)
    {
      GtkWidget* radio1;
      GtkWidget* radio2;
      
      radio1 = gtk_object_get_data(GTK_OBJECT(dialog), radio_key1);
      radio2 = gtk_object_get_data(GTK_OBJECT(dialog), radio_key2);
         
      /* If values for radio_key1 and radio_key2 was changed, but changes wasn't apply 
	 then no values have been stored into database for radio_key1 and radio_key2, 
	 so if user wants to revert, he can revert only to default value 
	 stored into schema.
       */
      if (value1 == NULL)
        {
	  /* We need to check for a default value,
             since the revert set will unset the user setting */	  
          GConfValue* def;  

          def = gconf_client_get_default_from_schema(client, radio_key1, NULL);  
	   
          if (def)  
            {  
              if (def->type == GCONF_VALUE_BOOL)  
                {  
		  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio1),  
						gconf_value_get_bool(def));   
                }   
              gconf_value_free(def); 
            }	   
        }
       else
	 if (value1->type == GCONF_VALUE_BOOL)
	   gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio1),
					 gconf_value_get_bool(value1));

      if (value2 == NULL)
	{
	 
          GConfValue* def; 

          def = gconf_client_get_default_from_schema(client, radio_key2, NULL); 

          if (def) 
            { 
              if (def->type == GCONF_VALUE_BOOL) 
                { 
		  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio2),
						gconf_value_get_bool(def)); 
                } 
              gconf_value_free(def); 
            }	   
	}
      else
	if (value2->type == GCONF_VALUE_BOOL)
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio2),
					gconf_value_get_bool(value2));           
    }
}




void rubrica_preferences_revert_checked_entry(GtkWidget *dialog, GConfChangeSet* cs, 
					      gchar* check_key, gchar* entry_key)
{
  GConfValue* value1 = NULL;
  GConfValue* value2 = NULL;
  GConfClient* client;
  GtkWidget* check;
  GtkWidget* gnome_entry;  
  GtkWidget *entry = NULL;
		  
  check       = gtk_object_get_data(GTK_OBJECT(dialog), check_key);
  gnome_entry = gtk_object_get_data(GTK_OBJECT(dialog), entry_key);
  client      = gtk_object_get_data(GTK_OBJECT(dialog), "client");

  if ((strcmp(entry_key, RUBRICA_KEY_BOOK) == 0))
    entry = gtk_object_get_data(GTK_OBJECT(dialog), "entry_addbook");  
  else
    entry = gtk_object_get_data(GTK_OBJECT(dialog), "entry_folder");
  
  if (gconf_change_set_check_value(cs, check_key, &value1))
    {    
      /* If values for check_key and entry_key was changed, but changes wasn't apply 
	 then no values have been stored into database for check_key and entry_key, 
	 so if user wants to revert, he can revert only to default value 
	 stored into schema.
      */         
      if (value1 == NULL)
	value1 = gconf_client_get_default_from_schema(client, check_key, NULL);

      if (value1 && value1->type == GCONF_VALUE_BOOL)
	{	  
	  gboolean bool;
	  
	  bool = gconf_value_get_bool(value1);
	  
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check),bool);
	  gtk_widget_set_sensitive(entry, bool);  
	}
    }
  
  if(gconf_change_set_check_value(cs, entry_key, &value2))
    {
      if (value2 == NULL)
	/* get default from schema */
	value2 = gconf_client_get_default_from_schema(client, entry_key, NULL);
      
      if (value2 && value2->type == GCONF_VALUE_STRING)
	gtk_entry_set_text(GTK_ENTRY(entry), gconf_value_get_string(value2));      
      else
	gtk_entry_set_text(GTK_ENTRY(entry), "");
    }
}  


void rubrica_preferences_revert_spin_button(GtkWidget *dialog, GConfChangeSet* cs, 
					    gchar* check_key, gchar* spin_key)
{
  GConfValue* checkvalue = NULL;
  GConfValue* spinvalue = NULL;  
  GConfClient* client;
  GtkWidget *check;
  GtkWidget* spin;

  check  = gtk_object_get_data(GTK_OBJECT(dialog), check_key);
  spin   = gtk_object_get_data(GTK_OBJECT(dialog), spin_key);  
  client = gtk_object_get_data(GTK_OBJECT(dialog), "client");

  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);

  if (gconf_change_set_check_value(cs, check_key, &checkvalue))
    {     
      if (checkvalue == NULL)
	checkvalue = gconf_client_get_default_from_schema(client, check_key, NULL);
      
      if (checkvalue && checkvalue->type == GCONF_VALUE_BOOL)
	{
	  gboolean bool;
	  
	  bool = gconf_value_get_bool(checkvalue);
	  
	  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(check), bool);      
	  gtk_widget_set_sensitive(spin, bool);
	}
    }
      
  if (gconf_change_set_check_value(cs, spin_key, &spinvalue))
        {
	  gint rate;
	  
	  if (spinvalue == NULL)
	    spinvalue = gconf_client_get_default_from_schema(client, spin_key, NULL);

	  if (spinvalue && spinvalue->type == GCONF_VALUE_INT)
	    {
	      rate = gconf_value_get_int(spinvalue);
	      
	      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), rate);

	      if (spin_key == RUBRICA_KEY_RATE)
		xmlSetCompressMode(rate);
	    }	  
	  else
	    {
	      gtk_spin_button_set_value(GTK_SPIN_BUTTON(spin), 0); 	  

	      if (spin_key == RUBRICA_KEY_RATE)
		xmlSetCompressMode(0);      
	    }	  
	}
}


void rubrica_preferences_revert_entry(GtkWidget *dialog, GConfChangeSet* cs, 
				      gchar* entry_key)
{
  GConfValue* value = NULL;
  GConfClient* client;
  
  client = gtk_object_get_data(GTK_OBJECT(dialog), "client");
  
  if (gconf_change_set_check_value(cs, entry_key, &value))
    {
      GtkWidget* entry;
    
      entry = gtk_object_get_data(GTK_OBJECT(dialog), entry_key);     
     
      if (value == NULL)
	value = gconf_client_get_default_from_schema(client, entry_key, NULL);
      
      if (value && value->type == GCONF_VALUE_STRING)
	{
	  gtk_entry_set_text(GTK_ENTRY(entry), gconf_value_get_string(value));
	  gconf_value_free(value);      
	}
      else
	gtk_entry_set_text(GTK_ENTRY(entry), "");      
    }
}


void rubrica_preferences_revert_combo(GtkWidget *dialog, GConfChangeSet* cs, 
				      gchar* entry_key)
{
  GConfValue* value = NULL;
  GConfClient* client;
  gchar *str = RUBRICA_SEARCH_CARD;;
  
  client = gtk_object_get_data(GTK_OBJECT(dialog), "client");
  
  if (gconf_change_set_check_value(cs, entry_key, &value))
    {
      GtkWidget* entry;
    
      entry = gtk_object_get_data(GTK_OBJECT(dialog), entry_key);     
     
      if (value == NULL)
	value = gconf_client_get_default_from_schema(client, entry_key, NULL);
      
      if (value && value->type == GCONF_VALUE_INT)
	{
	  gint type;

	  type = gconf_value_get_int(value);

	  switch (type)
	    {
	    case 0:
	      str = RUBRICA_SEARCH_CARD;
	      break;
	      
	    case 1:
	      str = RUBRICA_SEARCH_FIRST_NAME;
	      break;
	      
	    case 2:
	      str = RUBRICA_SEARCH_LAST_NAME;
	      break;
	      
	    case 3:
	      str = RUBRICA_SEARCH_PROFESSION;
	      break;
	      
	    case 4:
	      str = RUBRICA_SEARCH_CITY;
	      break;
	      
	    case 5:
	      str = RUBRICA_SEARCH_COUNTRY; 
	      break;
	      
	    case 6:
	      str = RUBRICA_SEARCH_WEB_ADDRESS;
	      break;
	      
	    case 7: 
	      str = RUBRICA_SEARCH_EMAIL_ADDRESS;
	      break;
	  
	    case 8:
	      str = RUBRICA_SEARCH_TELEPHONE;
	      break;
	      
	    default:
	      str = RUBRICA_SEARCH_CARD;
	      break;	      
	    }

	  gconf_value_free(value);      
	}     
      /* else */
      gtk_entry_set_text(GTK_ENTRY(entry), str);
    }
}
